/*
	sample for select: simplified telnet client.
*/
#include <stdio.h>
#include <netdb.h>
#include <termios.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>

main( int argc, char *argv[])

{	

        struct termios tt;

	fd_set rdfds;			/* file descriptors array for select */
	int numfds;			/* number of file descriptors */

	char wont[3]="\377\374"; 	/* <IAC> + <WON'T> */

	int			sock, addrlen;		/* socket stuff */
	struct sockaddr_in	addr;
	struct hostent 		*host;

	char c;				/* universal character buffer */
	
	if(argc != 2){
		printf("usage: tn hostname\n");
		exit(-1);
	}
	
	/* 
	establish a client socket on telnet port to remote host
	*/
	
	if( (sock = socket(AF_INET, SOCK_STREAM, 0)) <0){
		perror("socket");
		exit(1);
	}
	
	bzero((char *) &addr, sizeof(addr));
	addr.sin_family 	=AF_INET;
	addr.sin_port		=htons(23);
	
	/*
	try to resolve host address: first try dotted decimal, second try name
	*/
	if((addr.sin_addr.s_addr=inet_addr(argv[1])) ==-1){
		if((host=gethostbyname(argv[1])) ==NULL){
			printf("host %s unknown\n", argv[1]);
			exit(-1);
		}else bcopy(host->h_addr, &addr.sin_addr, host->h_length);
	}

	if(connect(sock, (struct sockaddr *)&addr, sizeof(addr)) <0){
		perror("connect");
		exit(1);
	}
	
	/*
	we are connected to telnet client:
	monitor tty and socket, passing data between them
	*/
	FD_ZERO(&rdfds);		/* initialize file descriptors array */
	FD_SET(0, &rdfds);
	FD_SET(sock, &rdfds);
	numfds=sock+1;
	
	/* 
	put tty in RAW, NOECHO mode 
	*/
        tcgetattr(STDIN_FILENO, &tt);
        tt.c_lflag &= ~ICANON;
        tt.c_lflag &= ~ECHO;
        tcsetattr(STDIN_FILENO, TCSANOW, &tt);
	/*
	wait for characters from stdin or socket
	*/
	for(;;){
		if(select(numfds, &rdfds, (fd_set *)0, (fd_set *)0, NULL) <0){
			perror("select");
			exit(-1);
		}
		/*
		from stdin ? read and transmit on socket
		*/
		if(FD_ISSET(0, &rdfds)){
			read(0, &c, 1);
			write(sock, &c, 1);
		}
		/*
		from socket ? read and write on stdout
		*/
		if(FD_ISSET(sock, &rdfds)){
			if(read(sock, &c, 1) ==0){  /* <EOF> => exit */
				printf("\r\nconnection closed\r\n");
                                tt.c_lflag |= ICANON;
                                tt.c_lflag |= ECHO;
                                tcsetattr(STDIN_FILENO, TCSANOW, &tt);
				exit(0);
			}
			if(c =='\377'){             /* <IAC> => refuse any negociation */
				read(sock, &c, 1);
				read(sock, &c, 1);
				write(sock, wont, 2);
				write(sock, &c, 1);
			}else write(1, &c, 1);	    /* data => transmit */
		}
		/*
		restore file descriptors array and continue
		*/
		FD_SET(0,&rdfds);
		FD_SET(sock, &rdfds);
	}
	
}
